#!/usr/bin/env python
# 
# This script is used by apache in order to authenticate
# and authorize access to resources within the /data/web/nbc-bin/protected
# directory.
#
# @author pgifford

from mod_python import apache
import psycopg2
import commands


# This class encapsulates the column indexs of a row of the users table in postgres.
# Update this class if the users table schema changes.
class UsersTableCol:
  id=0            # used in this script
  username=1      # used in this script
  password=2      # used in this script
  enabled=3       # used in this script
  is_remote=4


# This class encapsulates the column indexs of a row of the user_authorities table in postgres.
# Update this class if the user_authorities table schema changes.
class UserAuthTableCol:
  user_id=0       # used in this script
  authority=1     # used in this script


# Internal authorization codes, later mapped to apache request result codes.
class Auth:
  PASSED=0
  USER_NOT_FOUND=1
  USER_NOT_ADMIN=2
  USER_DISABLED=3
  BAD_PW=4
  INCOMPLETE_CREDENTIALS=5
  DB_QUERY_ERROR=6


# Quick and dirty method to covert auth codes to nice names.
# I'm purposely making this method a non-member of Auth
# so I don't have to create an instance of Auth to use it.
def authToString(authCode):
  if authCode == Auth.USER_NOT_FOUND:
    result = "USER_NOT_FOUND"
  elif authCode == Auth.USER_NOT_ADMIN:
    result = "USER_NOT_ADMIN"
  elif authCode == Auth.USER_DISABLED:
    result = "USER_DISABLED"
  elif authCode == Auth.BAD_PW:
    result = "BAD_PW"
  elif authCode == Auth.PASSED:
    result = "PASSED"
  elif authCode == Auth.INCOMPLETE_CREDENTIALS:
    result = "INCOMPLETE_CREDENTIALS"
  elif authCode == Auth.DB_QUERY_ERROR:
    result = "DB_QUERY_ERROR"
  else:
    result = "NOT DEFINED"

  return result


# Map the internal result code to the apache request result code,
# that will hit the client browser.
def mapApacheCode(authCode):
  if authCode == Auth.PASSED:
    result = apache.OK
  elif authCode == Auth.USER_NOT_FOUND:
    result = apache.HTTP_UNAUTHORIZED
  elif authCode == Auth.USER_NOT_ADMIN:
    result = apache.HTTP_FORBIDDEN
  elif authCode == Auth.USER_DISABLED:
    result = apache.HTTP_FORBIDDEN
  elif authCode == Auth.BAD_PW:
    result = apache.HTTP_UNAUTHORIZED
  elif authCode == Auth.INCOMPLETE_CREDENTIALS:
    result = apache.HTTP_UNAUTHORIZED
  elif authCode == Auth.DB_QUERY_ERROR:
    result = apache.HTTP_INTERNAL_SERVER_ERROR
  else:
    result = apache.HTTP_UNAUTHORIZED

  return result


# This is the command-line to decrypt TEA encrypted passwords.
teaCmd = """java -cp /usr/local/netbotz/nbc/jars/central.jar:/usr/local/netbotz/nbc/jars/com.apc.isxc.util.jar com.netbotz.server.tools.TeaCalc -d %s"""


#
# checks to make sure the user is an admin and enabled
#
def userAuth(username, password):
  # Query db for username
  try:
    conn = psycopg2.connect("dbname='netbotz' user='root' host='localhost' password='onyx'")
    cursor = conn.cursor()
    cursor.execute("SELECT * from users where username = %s", (username,))
  except:
    print "Exception caught on users table access."
    return Auth.DB_QUERY_ERROR

  # Verify the user's password
  user = cursor.fetchone()
  if user != None:
    print "user: ", user

    # Decrypt pw from db and compare
    teapw = user[UsersTableCol.password] # TEA encrypted password from db
    clearpw = commands.getoutput(teaCmd % (teapw,))
    if password == clearpw:
      pass
    else:
      print "User:", username, "password incorrect."
      return Auth.BAD_PW
    
    # Password is good, check the role
    enabled = user[UsersTableCol.enabled]
    if enabled:
      try:
        cursor.execute("SELECT * from user_authorities where user_id = %s and authority = %s", (user[UsersTableCol.id],'ROLE_NBC_ADMIN',))
      except:
        print "Exception caught on user_authorities table access."
        return Auth.DB_QUERY_ERROR
      
      admin = cursor.fetchone()
      if admin != None:
        print "Role: ", admin[UserAuthTableCol.authority]
      else:
        print "User:", username, "is not an administrator."
        return Auth.USER_NOT_ADMIN
    else:
      print "User:", username, "is not enabled."
      return Auth.USER_DISABLED
  else:
    print "User not found:", username
    return Auth.USER_NOT_FOUND

  return Auth.PASSED



# Extract the username / password from the http request and call
# the authorize method.
def getAuth(req):
  password = req.get_basic_auth_pw() # This has to be called before using req.user
  username = req.user
  if username == None or password == None:
    code = Auth.INCOMPLETE_CREDENTIALS
  else:
    code = userAuth(username, password)

  if code != Auth.PASSED:
    req.log_error("Username: %s, authCode: %s" % (username, authToString(code)))
  
  return code


# This implements the authenhandler api for apache.
def authenhandler(req):
  """This function gets called by mod_python to handle Apache's authentication phase"""
  code = mapApacheCode(getAuth(req))
  return code


# This implements the authzhandler api for apache.
def authzhandler(req):
  """This function gets called by mod_python to handle Apache's authorization phase"""
  code = mapApacheCode(getAuth(req))
  return code


# The following can be uncommented to run from the command line as a test
#print userAuth("apc", "apc")
