#!/bin/sh
##################################
#
# Source this file for access to the ISXC plugins 
# shell api.
#
##################################


WEB_ROOT=/data/web
WEBAPPS=/etc/nbc/webapps
JARS=/usr/local/netbotz/nbc/jars/plugins
ISXC_HOME=/usr/local/isxc
PROXY_AJP=/etc/httpd/conf.d/proxy_ajp.conf
SCRIPT_DIR=/data/scripts

#
# Enable an ISXC plugin.
#
# args: <plugin-name>
# Where plugin name is a folder in /usr/local/isxc
#
enablePlugin() {
    pkg=$ISXC_HOME/$1
    
    mkdir -p $WEB_ROOT/plugins
    mkdir -p $JARS
    
    if [ -d $pkg/client ]
    then
        ln -s $pkg/client $WEB_ROOT/plugins/$1
    fi
    
    if [ -d $pkg/classpath ]
    then
        ln -s $pkg/classpath $JARS/$1
    fi
    
    for webapp in $pkg/webapps/*
    do
        name=`basename $webapp`

        ln -s $webapp $WEBAPPS/$name

        # Pass requests to servlet for processing
        if [[ ! `grep $name $PROXY_AJP` ]]; then
            echo "ProxyPass /$name/ ajp://localhost:8009/$name/ retry=15 flushpackets=auto" >> $PROXY_AJP
        fi
    done
    
    rm -f /data/tmp/.$1
    
    enableInitScript $1
    
    #only try to work with autoEnables if $2 is null
    if [ ! "$2" ]
    then
        enableAutoEnables $1
    fi
}

#
# Disable an ISXC plugin
#
# args: <plugin-name>
# Where plugin name is a folder in /usr/local/isxc
#
disablePlugin() {
    
    pkg=$ISXC_HOME/$1
    
    rm -f $WEB_ROOT/plugins/$1
    rm -f $JARS/$1
    
    for webapp in $pkg/webapps/*
    do
        name=`basename $webapp`

        rm -f $WEBAPPS/$name

        grep -v $name $PROXY_AJP > ${PROXY_AJP}.new
        cp -f ${PROXY_AJP}.new $PROXY_AJP
    done
    
    # Make sure java pluginsManagerImpl doesn't re-enable the plugin
    touch /data/tmp/.$1
    
    disableInitScript $1
    
    #only try to work with autoEnables if $2 is null
    if [ ! "$2" ]
    then
        disableAutoEnables $1
    fi
}

#
# Test if a plugin is enabled.  Look for a web client link, link to classpath 
# folder and any webapp.
#
isEnabled() {

    pkg=$ISXC_HOME/$1
    
    if [ -h $WEB_ROOT/plugins/$1 -o -h $JARS/$1 -o `ls -al $WEBAPPS | grep -c "$1"` -gt 0 ]
    then
        return 1
    fi
    return 0
}

#
# Disable all ISXC plugins
#
disableAllPlugins() {
    
	for file in `find /usr/local/isxc -name licenseContext.xml`
	do
		pluginDir=`dirname $file`
		pluginName=`basename $pluginDir`
	    disablePlugin $pluginName	
	done

}

#
# Disable InitScript for the given plugin.
#
# args: <plugin-name>
# Where: plugin-name is the name of the plugin in /usr/local/isxc
#
disableInitScript() {
	
    for depend in `getInitScript $1`
    do
        if [ $depend ] 
        then
            plugin=`echo $depend | sed -e "s/=.*//"`
            rm -f $SCRIPT_DIR/$plugin-init.sh
        fi
    done
    
}

#
# Enable InitScript for the given plugin.
#
# args: <plugin-name>
# Where: plugin-name is the name of the plugin in /usr/local/isxc
#
enableInitScript() {

    mkdir -p $SCRIPT_DIR
    for depend in `getInitScript $1`
    do
        if [ $depend ] 
        then
            script=`echo $depend | sed -e "s/.*=//"`
            plugin=`echo $depend | sed -e "s/=.*//"`
            ln -s $script $SCRIPT_DIR/$plugin-init.sh
        fi
    done
    
}

#
# Disable the autoenables for a given plugin if they are no longer used.
#
# args: <plugin-name>
# Where: plugin-name is a folder in /usr/local/isxc
#
disableAutoEnables() {
	
	# newline separated list of <pluginName>=<autoEnablePlugin>
	dependsToDisable=`getAutoEnables $1`

	if [ ! "$dependsToDisable" ]
	then
		return
	fi
	# For each plugin	
	for file in `find /usr/local/isxc -name licenseContext.xml`
    do
        pluginDir=`dirname $file`
        pluginName=`basename $pluginDir`
        # if the plugin is enabled
        `isEnabled $pluginName`
        if [ $? == 1 ]
        then
        	# Find the autoEnables of each enabled plugin
        	potentialDependPlugins=`getAutoEnables $pluginName`

        	for depend in $dependsToDisable
        	do
        		disable=`echo $depend | sed -e "s/.*=//"`

                # $plugin will be of the form plugin=autoEnablePlugin                
	            if [ `echo $potentialDependPlugins | grep "$disable"` ]
	            then
	            	dependsToDisable=`echo $dependsToDisable | grep -v $disable`
	            fi
        	done
        fi
    done
	
	for plugin in `echo $dependsToDisable`
	do
		disable=`echo $plugin | sed -e "s/.*=//"`
		
		disablePlugin $disable "auto"
	done
	
}

#
# Enable plugins that don't have their own license, but are specified by others.
#
enableAutoEnables() {

    for depend in `getAutoEnables $1`
    do
    	if [ $depend ] 
    	then
	        enable=`echo $depend | sed -e "s/.*=//"`
	        enablePlugin $enable "auto"
    	fi
    done

}

#
# Get the autoenables for a given plugin.
#
# args: <plugin-name>
# Where: plugin-name is the name of the plugin in /usr/local/isxc
#
# returns: newline separated list of plugin=autoEnable
#
getAutoEnables() {
	if [[ `grep autoEnable $ISXC_HOME/$1/licenseContext.xml` ]]
    then
		echo `perl -e '
		   # Loop the file looking for spring beans
		    while(<>) {
		        if( /<bean/ ) {
		            # Found a bean, go into it to find the name and look for auto enables
		            while(<>) {
		                chop;
		                if( /name="name"/ ) {
		                    # Save the plugin name
		                    s/.*value="//;
		                    s/".*//;
		                    $pluginName = $_;
		                }
		                
		                if( /autoEnable/ ) {
		                    # An Autoenable.  Pull out this chunk of xml and parse it
		                    $auto = $_;
		                    while( $auto !~ /\/property/ ) {
		                        $auto = $auto . <>;
		                    }
		                    # Strip off all the xml and print each autoenable plugin
		                    $auto =~ s/\n//g;
		                    $auto =~ s/.*<list//g;
		                    $auto =~ s/<\/list.*//g;
		                    @auto = split( /\/value/, $auto );
		                    foreach $plugin (@auto) {
		                        $plugin =~ s/^.*value>//;
		                        $plugin =~ s/[<>]*//g;
		                        if( $plugin !~ /^[ \t]*$/ ) {
		                            # <pluginName>=<autoEnablePluginName>
		                            print "$pluginName=$plugin\n";
		                        }
		                    }
		                }
		                # End of this bean, return to the top to look for the next
		                if( /\/bean/ ) {
		                    last;
		                }
		            }
		        }
		    }
		' $ISXC_HOME/$1/licenseContext.xml` 
    fi
}

#
# Get the initscript for the given plugin.
#
# args: <plugin-name>
# Where: plugin-name is the name of the plugin in /usr/local/isxc
#
getInitScript() {
    if [[ `grep initScript $ISXC_HOME/$1/licenseContext.xml` ]]
    then
        echo `perl -e '
           # Loop the file looking for spring beans
            while(<>) {
                if( /<bean/ ) {
                    # Found a bean, go into it to find the name and look for initScript
                    while(<>) {
                        chop;
                        if( /name="name"/ ) {
                            # Save the plugin name
                            s/.*value="//;
                            s/".*//;
                            $pluginName = $_;
                        }
                        
                        if( /initScript/ ) {
                            # An initScript.  Pull out this chunk of xml and parse it
                            $auto = $_;
                            while( $auto !~ /\/property/ && $auto !~ /\/>/ ) {
                                $auto = $auto . <>;
                            }
                            # Strip off all the xml and print each autoenable plugin
                            $auto =~ s/\n//g;
                            $auto =~ s/.*value="//g;
                            $auto =~ s/".*//;
                            print "$pluginName=$auto\n";
                        }
                        # End of this bean, return to the top to look for the next
                        if( /\/bean/ ) {
                            last;
                        }
                    }
                }
            }
        ' $ISXC_HOME/$1/licenseContext.xml` 
    fi
}


#
# Main Execution
#
case "$1" in
  enable)
        enablePlugin $2
        ;;
  disable)
        disablePlugin $2
        ;;
  isEnabled)
        isEnabled $2
        ;;
  getAutoEnables)
        getAutoEnables $2
        ;;
  enableInitScript)
        enableInitScript $2
        ;;
  disableInitScript)
        disableInitScript $2
        ;;
  *)
        echo $"Usage: $0 {enable|disable|isEnabled}"
        exit 1
esac

